﻿using System.Windows.Shapes;
using System.Windows.Controls;
using MazeGenerator;
using MazeGenerator.MazeGenerator;


/// <summary>
/// Represents one maze cell on drawing board
/// </summary>

class MazeDrawerCell
{
    /// <summary>
    /// references to all four edges
    /// </summary>
    private MazeDrawerEdge[] edges;

    /// <summary>
    /// reference to graphic rectangle object of this cell
    /// </summary>
    private Rectangle rectangle;

    /// <summary>
    /// Cell constructor.
    /// </summary>
    public MazeDrawerCell()
    {
        edges = new MazeDrawerEdge[4];
        rectangle = new Rectangle();
    }

    /// <summary>
    /// Cell constructor, if all four edges already exist.
    /// </summary>
    /// <param name="northEdge"></param>
    /// <param name="eastEdge"></param>
    /// <param name="southEdge"></param>
    /// <param name="westEdge"></param>
    public MazeDrawerCell(MazeDrawerEdge northEdge, MazeDrawerEdge eastEdge, MazeDrawerEdge southEdge, MazeDrawerEdge westEdge)
        : this()
    {
        edges[MazeDir.NORTH.GetInt()] = northEdge;
        edges[MazeDir.EAST.GetInt()] = eastEdge;
        edges[MazeDir.SOUTH.GetInt()] = southEdge;
        edges[MazeDir.WEST.GetInt()] = westEdge;
    }

    /// <summary>
    /// Set edge at given position
    /// </summary>
    /// <param name="dir"></param>
    /// <param name="edge"></param>
    public void SetEdge(MazeDir dir, MazeDrawerEdge edge)
    {
        edges[dir.GetInt()] = edge;
    }

    /// <summary>
    /// Get edge from given position
    /// </summary>
    /// <param name="dir"></param>
    /// <returns></returns>
    public MazeDrawerEdge GetEdge(MazeDir dir)
    {
        return edges[dir.GetInt()];
    }

    /// <summary>
    /// Set the graphical position of a edge.
    /// </summary>
    /// <param name="dir"></param>
    /// <param name="x1"></param>
    /// <param name="y1"></param>
    /// <param name="x2"></param>
    /// <param name="y2"></param>
    public void SetEdgePos(MazeGenerator.MazeGenerator.MazeDir dir, int x1, int y1, int x2, int y2)
    {
        edges[dir.GetInt()].SetEdgePos(x1, y1, x2, y2);
    }

    /// <summary>
    /// Set the bounding box of the rectangle.
    /// </summary>
    /// <param name="x"></param>
    /// <param name="y"></param>
    /// <param name="width"></param>
    /// <param name="height"></param>
    public void SetCellRectanglePos(int x, int y, int width, int height)
    {
        rectangle.SetValue(Canvas.LeftProperty, (double)x);
        rectangle.SetValue(Canvas.TopProperty, (double)y);
        rectangle.Width = width;
        rectangle.Height = height;
    }

    /// <summary>
    /// Get the graphical object of this rectangle.
    /// </summary>
    /// <returns></returns>
    public Rectangle GetCellRectangle()
    {
        return rectangle;
    }


}