﻿using System.Windows.Media;
using System.Windows.Shapes;

/// <summary>
/// Represents one edge of a maze cell. This edge might be referenced by one or two adjacent cells.
/// </summary>

class MazeDrawerEdge
{
    /// <summary>
    /// graphical respresentation of edge
    /// </summary>
    private Line line;

    public static readonly SolidColorBrush wallEdgeColorBrush = new SolidColorBrush(Colors.Blue);

    /// <summary>
    /// Constructor of drawable edge.
    /// </summary>
    public MazeDrawerEdge()
    {
        line = new Line();
        line.Stroke = wallEdgeColorBrush;
        line.StrokeThickness = 3;
    }

    /// <summary>
    /// Set position of edge relative to parent container of first endpoint. 
    /// </summary>
    /// <param name="x1">x-position in pixel relative to parent container of first endpoint</param>
    /// <param name="y1">y-position in pixel relative to parent container of first endpoint</param>
    /// <param name="x2">x-position in pixel relative to parent container of second endpoint</param>
    /// <param name="y2">y-position in pixel relative to parent container of second endpoint</param>
    public void SetEdgePos(int x1, int y1, int x2, int y2)
    {
        line.X1 = x1;
        line.Y1 = y1;
        line.X2 = x2;
        line.Y2 = y2;
    }

    /// <summary>
    /// Get the line edge.
    /// </summary>
    /// <returns></returns>
    public Line GetEdgeLine() { return line; }

}