﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Collections.Generic;

namespace MazeGenerator.MazeGenerator.MazeGeneratorAlgorithm
{
    public class MazeGenDFS : AbstractImmediateMazeGenerator
    {
        /// <summary>
        /// stack holding all cells to be visited. Because there is no "cell object" accessible to be used for the stack, 
        /// we use a tuple with the x and y-coordinate of the cell.
        /// </summary>
        Stack<Tuple<int, int>> stack;

        public MazeGenDFS(int width, int height) : base(width, height) { }

        public MazeGenDFS(int width, int height, int startCellX, int startCellY) : base(width, height, startCellX, startCellY) { }

        public override void Generate()
        {
            stack = new Stack<Tuple<int, int>>();
            // start at topleft cell
            stack.Push(Tuple.Create<int, int>(0, 0));
            VisitCell(StartCellX, StartCellY);

            while (stack.Count > 0)
            {
                Tuple<int, int> cellPos = stack.Pop();

                MazeDir[] randMazeDirOrder = GetRandomMazeDirOrder();
                foreach (MazeDir dir in randMazeDirOrder)
                {
                    if (ExistsWall(cellPos.Item1, cellPos.Item2, dir) && IsAdjacentCellVisited(cellPos.Item1, cellPos.Item2, dir) == false)
                    {
                        RemoveWall(cellPos.Item1, cellPos.Item2, dir);
                        Tuple<int, int> adjCell = Tuple.Create<int, int>(cellPos.Item1 + DELTA_X[(ushort)dir],
                                                                         cellPos.Item2 + DELTA_Y[(ushort)dir]);
                        VisitCell(adjCell.Item1, adjCell.Item2);
                        stack.Push(adjCell);
                    }
                }
            }
        }
    }
}
