﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using MazeGenerator.MazeGenerator.MazeGeneratorAlgorithm;

namespace MazeGenerator.MazeGenerator
{
    /// <summary>
    /// A Depth-first search algorithm implemented with recursion.
    /// </summary>
    public class MazeGenDFSRecursive : AbstractImmediateMazeGenerator
    {
        public MazeGenDFSRecursive(int width, int height) : base(width, height) { }
        public MazeGenDFSRecursive(int width, int height, int startCellX, int startCellY) :
            base(width, height, startCellX, startCellY) { }

        public override void Generate()
        {
            Generate(StartCellX, StartCellY); // start at top left cell. Could be improved.
        }

        private void Generate(int x, int y)
        {
            // at first mark this cell as visited
            VisitCell(x, y);
            // get the four possible directions in random order to move to a random next cell.
            MazeDir[] randMazeDirOrder = GetRandomMazeDirOrder();
            
            foreach (MazeDir dir in randMazeDirOrder)
            {
                if (ExistsWall(x, y, dir) && IsAdjacentCellVisited(x, y, dir) == false)
                {   // there is wall and the adjacent cell was not visited, so here we can go on.
                    // remove wall, go to adjacent cell and continue
                    RemoveWall(x, y, dir);
                    Generate(x + DELTA_X[(ushort)dir], y + DELTA_Y[(ushort)dir]);
                }
            }
        }
    }
}
