﻿using MazeGenerator.MazeGenerator;
using System;
using System.Collections.Generic;
using MazeGenerator.MazeGenerator.MazeGeneratorAlgorithm;


class MazeGenHuntAndKill : AbstractImmediateMazeGenerator
{

    public MazeGenHuntAndKill(int width, int height) : base(width, height) { }
    public MazeGenHuntAndKill(int width, int height, int startCellX, int startCellY) :
        base(width, height, startCellX, startCellY) { }

    public override void Generate()
    {
        Tuple<int, int> curCell = Tuple.Create<int, int>(0, 0);
        VisitCell(StartCellX, StartCellY);

        while (curCell != null)
        {
            walk(curCell);
            curCell = hunt();
        }
    }

    private void walk(Tuple<int, int> startCell)
    {
        Tuple<int, int> curCell = startCell;
        
        bool deadEnd = false;
        while (!deadEnd)
        {
            deadEnd = true;
            MazeDir[] randMazeDirOrder = GetRandomMazeDirOrder();
            foreach (MazeDir dir in randMazeDirOrder)
            {
                if (ExistsWall(curCell.Item1, curCell.Item2, dir) &&
                    IsAdjacentCellVisited(curCell.Item1, curCell.Item2, dir) == false)
                {
                    RemoveWall(curCell.Item1, curCell.Item2, dir);
                    Tuple<int, int> adjCell = Tuple.Create<int, int>(curCell.Item1 + DELTA_X[(ushort)dir],
                                                                     curCell.Item2 + DELTA_Y[(ushort)dir]);
                    VisitCell(adjCell.Item1, adjCell.Item2);
                    curCell = adjCell;
                    deadEnd = false;
                    break;
                }
            }
        }
    }

    private Tuple<int, int> hunt()
    {
        for (int x = 0; x < this.Width; x++)
        {
            for (int y = 0; y < this.Height; y++)
            {
                if (!IsCellVisited(x, y))
                {
                    /* cell not visited, check if any adjacent cell is already visited */
                    MazeDir[] randMazeDirOrder = GetRandomMazeDirOrder();
                    foreach (MazeDir dir in randMazeDirOrder)
                    {
                        if (IsAdjacentCellInsideMaze(x, y, dir) &&
                            IsAdjacentCellVisited(x, y, dir) )
                        {
                            /* found, return the found adjacent cell */
                            return Tuple.Create<int, int>(x + DELTA_X[(ushort)dir], y + DELTA_Y[(ushort)dir]);
                        }
                    }
                }
            }
        }
        return null;
    }
}