﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Collections.Generic;
using System.Threading;
using System.Windows.Threading;

namespace MazeGenerator.MazeGenerator.StepwiseMazeGeneratorAlgorithm
{
    /// <summary>
    /// Step-by-step maze calculation of a maze using a depth first search.
    /// </summary>
    public class StepwiseMazeGenDFS : AbstractStepwiseMazeGenerator
    {
        private Stack<Tuple<int, int>> stack;

        /// <summary>
        /// Constructor.
        /// </summary>
        /// <param name="width"></param>
        /// <param name="height"></param>
        /// <param name="parent"></param>
        /// <param name="startCellX"></param>
        /// <param name="startCellY"></param>
        public StepwiseMazeGenDFS(int width, int height, UserControl parent, int startCellX, int startCellY)
            : base(width, height, parent, startCellX, startCellY)
        {
            // init maze
            stack = new Stack<Tuple<int, int>>();
            // start at topleft cell
            stack.Push(Tuple.Create<int, int>(startCellX, startCellY));
            VisitCell(startCellX, startCellY);
        }

        /// <summary>
        /// Constructor.
        /// </summary>
        /// <param name="width"></param>
        /// <param name="height"></param>
        /// <param name="parent"></param>
        public StepwiseMazeGenDFS(int width, int height, UserControl parent)
            : this(width, height, parent, -1, -1) { }

        protected override void DoWork()
        {
            WaitAndNotify(false);

            while (stack.Count > 0)
            {
                Tuple<int, int> cellPos = stack.Pop();

                MazeDir[] randMazeDirOrder = GetRandomMazeDirOrder();
                foreach (MazeDir dir in randMazeDirOrder)
                {
                    if (ExistsWall(cellPos.Item1, cellPos.Item2, dir) && IsAdjacentCellVisited(cellPos.Item1, cellPos.Item2, dir) == false)
                    {
                        RemoveWall(cellPos.Item1, cellPos.Item2, dir);
                        Tuple<int, int> adjCell = Tuple.Create<int, int>(cellPos.Item1 + DELTA_X[(ushort)dir],
                                                                         cellPos.Item2 + DELTA_Y[(ushort)dir]);
                        VisitCell(adjCell.Item1, adjCell.Item2);
                        stack.Push(adjCell);
                    }
                }
                WaitAndNotify(false);
            }

            WaitAndNotify(true);
        }        
    }
}